// Package model provides model abstraction for cardpayment service
package model

import (
	"context"

	"gitlab.mashplace.com/mashplace/backend/common/api/generic/query"

	"go.mongodb.org/mongo-driver/bson/primitive"
)

// Repository provides persistent operations for cardpayment service
type Repository interface {
	Migrate(ctx context.Context) error
	ChargeSave(ctx context.Context, charge *Charge) error
	ChargeRemove(ctx context.Context, id string) error
	ChargeGetByID(ctx context.Context, id string) (*Charge, error)
	ChargeGetBySessionID(ctx context.Context, sessionID string) (*Charge, error)
	ChargeGetBySubscriptionID(ctx context.Context, subid string) (*Charge, error)
	MandateSave(ctx context.Context, mandate *Mandate) error
	MandateRemove(ctx context.Context, id string) error
	MandateGetByID(ctx context.Context, id primitive.ObjectID) (*Mandate, error)
	MandateGetBySetupIntentID(ctx context.Context, setupIntentID string) (*Mandate, error)
	MandateGetByMandateID(ctx context.Context, mandateID string) (*Mandate, error)
	MandateGetByUserIDEmail(ctx context.Context, userID, email string) (*Mandate, error)
	MandateGetByCustomerID(ctx context.Context, customerID string) (*Mandate, error)
	FilterMandates(ctx context.Context, params *query.QueryParams) ([]*Mandate, int, error)
	UserOffersSave(ctx context.Context, u *UserOffers) error
	UserOffersGet(ctx context.Context, offer string) (*UserOffers, error)
	UserOffersGetByUser(ctx context.Context, user string) (models []*UserOffers, count int, err error)
	FilterUserOffers(ctx context.Context, params *query.QueryParams) (models []*UserOffers, count int, err error)
	FilterCharges(ctx context.Context, params *query.QueryParams) (models []*Charge, count int, err error)
	ChangelogSave(ctx context.Context, changelog *Changelog) error
}
