package model

import (
	"time"

	"gitlab.mashplace.com/mashplace/backend/common/generic/util"

	"go.mongodb.org/mongo-driver/bson/primitive"

	cardpayment "gitlab.mashplace.com/mashplace/backend/cardpayment/api"
)

// Mandate represents stripe mandate
type Mandate struct {
	ID              primitive.ObjectID        `bson:"_id"`
	SessionID       string                    `bson:"session_id"`
	SetupIntentID   string                    `bson:"setup_intent_id"`
	CustomerID      string                    `bson:"customer_id"`
	UserID          string                    `bson:"user_id"`
	Offers          []string                  `bson:"offers"`
	MandateID       string                    `bson:"mandate_id"`
	PaymentMethodID string                    `bson:"payment_method_id"`
	Email           string                    `bson:"email"`
	Status          cardpayment.MandateStatus `bson:"status"`
	Valid           bool                      `bson:"valid"`
	CreatedAt       time.Time                 `bson:"created_at"`
	UpdatedAt       time.Time                 `bson:"updated_at"`
}

// User ...
type User struct {
	ID    string `bson:"id"`
	Email string `bson:"email"`
}

// UserOffers represents users who have requested the mandate of the offer
type UserOffers struct {
	Offer string `bson:"offer"`
	Users []User `bson:"users"`
}

// RenderMandate converts model.Mandate into gRPC API data.
func (model *Mandate) RenderMandate() *cardpayment.Mandate {
	return &cardpayment.Mandate{
		Id:        model.ID.Hex(),
		MandateId: model.MandateID,
		Offers:    model.Offers,
		User:      model.UserID,
		Email:     model.Email,
		Status:    model.Status,
		Valid:     model.Valid,
		CreatedAt: util.ToUnixTime(model.CreatedAt),
		UpdatedAt: util.ToUnixTime(model.UpdatedAt),
	}
}
