// Package v1 provides migration to version 1 of mongo repository
package v1

import (
	"context"
	"time"

	cardpayment "gitlab.mashplace.com/mashplace/backend/cardpayment/api"
	"gitlab.mashplace.com/mashplace/backend/common/generic/log"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
	"go.mongodb.org/mongo-driver/mongo"
)

type charge struct {
	Items         []map[string]interface{}  `bson:"items,omitempty"`
	CreatedAt     time.Time                 `bson:"created_at,omitempty"`
	UpdatedAt     time.Time                 `bson:"updated_at,omitempty"`
	ID            string                    `bson:"_id,omitempty"`
	ClientSecret  string                    `bson:"client_secret,omitempty"`
	UserID        string                    `bson:"user_id,omitempty"`
	UserEmail     string                    `bson:"user_email,omitempty"`
	Property      string                    `bson:"property,omitempty"`
	NotifyID      string                    `bson:"notify_id"`
	NotifyAddress string                    `bson:"notify_address"`
	InvoiceNumber string                    `bson:"invoice_number"`
	InvoiceID     string                    `bson:"invoice_id"`
	SessionID     string                    `bson:"session_id,omitempty"`
	Total         int64                     `bson:"total,omitempty"`
	Type          cardpayment.PaymentType   `bson:"type"`
	Status        cardpayment.PaymentStatus `bson:"status,omitempty"`
	Transferred   bool                      `bson:"transferred"`
}

// Up applies v1 migration
func Up(ctx context.Context, db *mongo.Database, migrationID string) (int, error) {
	col := db.Collection("charges")

	cur, err := col.Find(ctx, bson.M{})
	if err != nil {
		return -1, err
	}

	var c charge

	for cur.Next(ctx) {
		err = cur.Decode(&c)
		if err != nil {
			return -1, err
		}

		for _, i := range c.Items {
			var itemID string

			if s, ok := i["_id"].(string); ok {
				itemID = s
			}

			var oid primitive.ObjectID

			if id, ok := i["item_id"].(string); ok {
				oid, _ = primitive.ObjectIDFromHex(id)
			}

			if oid.IsZero() {
				oid = primitive.NewObjectID()
			}

			i["_id"] = oid

			i["item_id"] = itemID
		}

		_, err = col.ReplaceOne(ctx, bson.M{"_id": c.ID}, c)
		if err != nil {
			log.Error(err)
		}
	}

	return -1, err
}
