package service

import (
	cardpayment "gitlab.mashplace.com/mashplace/backend/cardpayment/api"
	"gitlab.mashplace.com/mashplace/backend/cardpayment/internal/model"
	extras "gitlab.mashplace.com/mashplace/backend/extras/api"
	"google.golang.org/protobuf/types/known/timestamppb"
)

func renderService(m *model.Service) *extras.ServiceSnapshot {
	created := timestamppb.New(m.Created)
	updated := timestamppb.New(m.Updated)

	return &extras.ServiceSnapshot{
		Id:            m.ItemID,
		ServiceId:     m.ServiceID,
		Created:       created,
		Updated:       updated,
		Code:          m.Code,
		Type:          m.Type,
		Role:          m.Role,
		Price:         m.Price,
		TaxType:       m.TaxType,
		TaxAmount:     m.TaxAmount,
		TaxPercentage: m.TaxPercentage,
		Title:         m.Title,
		Description:   m.Description,
		Primary:       m.Primary,
	}
}

func renderCharge(m *model.Charge) *cardpayment.Receipt {
	var items []*extras.ServiceSnapshot
	for _, s := range m.Items {
		items = append(items, renderService(s))
	}

	created := timestamppb.New(m.CreatedAt)
	updated := timestamppb.New(m.UpdatedAt)

	if m.Type == cardpayment.PaymentType_UNKNOWN_PAYMENT_TYPE {
		m.Type = cardpayment.PaymentType_CARD
	}

	return &cardpayment.Receipt{
		Id:           m.ID,
		Type:         m.Type,
		ClientSecret: m.ClientSecret,
		UserId:       m.UserID,
		UserEmail:    m.UserEmail,
		Items:        items,
		Property:     m.Property,
		Status:       m.Status,
		CreatedAt:    created,
		UpdatedAt:    updated,
	}
}
